#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

"""
simple_augmented.py - Demonstrates how to call WebKit's JavaScript objects from
                      Python and call Python objects from JavaScript.

This file is part of the PyQt for Desktop and Embedded Devices package.

Copyright (C) 2009 David Boddie <david.boddie@nokia.com>
Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
All rights reserved.

Contact: Nokia Corporation (qt-info@nokia.com)

You may use this file under the terms of the BSD license as follows:

"Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor the names
   of its contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
"""

import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtWebKit import *


js = \
"""findAnchors = function()
{
    findAnchorsInNode(document, 0);
}

findAnchorsInNode = function(parent)
{
    var children = parent.childNodes;
    
    for (var i = 0; i < children.length; i++) {
        var child = children[i];
        if (child.nodeName == "A") {
            if (child.hasAttributes()) {
                var attributes = child.attributes;
                for (var j = 0; j < attributes.length; j++) {
                    if (attributes[j].nodeName == "name") {
                        var following = findFollowing(child);
                        anchorList.addHeading(following.nodeName, attributes[j].nodeValue, getAnchorText(following));
                    }
                }
            }
        } else if (child.hasChildNodes()) {
            findAnchorsInNode(child);
        }
    }
}

findFollowing = function(anchor)
{
    var next = anchor.nextSibling;
    while (next && next.nodeType != Node.ELEMENT_NODE)
        next = next.nextSibling;
    
    return next;
}

getAnchorText = function(parent)
{
    var children = parent.childNodes;
    var text = "";
    
    for (var i = 0; i < children.length; i++) {
        var child = children[i];
        if (child.nodeType == Node.TEXT_NODE)
            text = text + child.nodeValue;
        else if (child.hasChildNodes())
            text = text + findAnchorText(child);
    }
    
    return text;
}
"""

class Browser(QSplitter):

    default_url = QUrl("http://doc.qtsoftware.com/4.5/qt4-5-intro.html")
    
    def __init__(self, parent = None):
    
        QSplitter.__init__(self, Qt.Horizontal, parent)
        
        self.headingsList = QTreeView()
        self.headingsList.setHeaderHidden(True)
        self.headingsModel = QStandardItemModel()
        self.headingsList.setModel(self.headingsModel)
        
        self.webview = QWebView()
        
        self.connect(self.headingsList, SIGNAL("clicked(QModelIndex)"), self.goToAnchor)
        self.connect(self.webview, SIGNAL("titleChanged(QString)"), self.setWindowTitle)
        self.connect(self.webview, SIGNAL("loadFinished(bool)"), self.prepareJavaScript)
        
        self.addWidget(self.headingsList)
        self.addWidget(self.webview)
        self.setCollapsible(0, True)
        self.newDocument()
    
    def newDocument(self, url = default_url):
    
        self.webview.setUrl(url)
    
    def openUrl(self):
    
        text = self.addressEdit.text()
        self.webview.setUrl(QUrl(text))
    
    def prepareJavaScript(self, ready):
    
        if not ready:
            return
        
        self.webview.page().mainFrame().addToJavaScriptWindowObject("anchorList", self)
        self.webview.page().mainFrame().evaluateJavaScript(js)
        self.listHeadings()
    
    def listHeadings(self):
    
        self.headingsModel.clear()
        self.webview.page().mainFrame().evaluateJavaScript("findAnchors();")
    
    @pyqtSignature("addHeading(QString, QString, QString)")
    def addHeading(self, nodeName, name, title):
    
        if str(nodeName) not in ("H1", "H2", "H3", "H4", "H5", "H6"):
            return
        
        url = self.webview.page().mainFrame().url()
        url.setFragment(name)
        
        level = int(str(nodeName)[1])
        newItem = QStandardItem(title)
        newItem.setData(QVariant(url), Qt.UserRole)
        newItem.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
        
        parent = self.headingsModel
        i = 1
        while i < level:
            if parent.rowCount() == 0:
                item = QStandardItem("[%i]" % i)
                item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
                parent.appendRow(item)
            elif parent == self.headingsModel:
                item = parent.item(parent.rowCount() - 1, 0)
            else:
                item = parent.child(parent.rowCount() - 1, 0)
            parent = item
            self.headingsList.setExpanded(parent.index(), True)
            i += 1
        
        parent.appendRow(newItem)
    
    def goToAnchor(self, index):
    
        url = index.data(Qt.UserRole).toUrl()
        if url.isValid():
            self.webview.load(url)


if __name__ == "__main__":

    app = QApplication(sys.argv)
    browser = Browser()
    browser.show()
    sys.exit(app.exec_())
