"""
slides.py - Presentation contents and utility functions.
This file is part of the PyQt for Desktop and Embedded Devices package.

Copyright (C) 2009 David Boddie <david.boddie@nokia.com>
Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
All rights reserved.

Contact: Nokia Corporation (qt-info@nokia.com)

You may use this file under the terms of the BSD license as follows:

"Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor the names
   of its contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
"""

import math
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from items import *

# Utility functions

def create_list_items(list_items, position, scene, start = -1.0, finish = 1.0, final_opacity = 1.0):

    interval = 1.0 / len(list_items)
    items = []
    colors = (QColor(32, 96, 32), QColor(32, 32, 96), QColor(96, 32, 32))
    i = 0
    
    for text in list_items:
    
        t = text.lstrip()
        if t.startswith(u"* "):
            text = (len(text) - len(t)) * u" " + u"\u2022 " + t[2:]
        
        pairs = {start: 0.0, start + interval: 1.0, finish: final_opacity}
        if start + interval < 0.0:
            pairs[0.0] = 1.0
        pairs = pairs.items()
        pairs.sort()
        
        item = FadingLabel(text, pairs, scene)
        item.setFont(List_Font)
        item.setColor(colors[i % 3])
        item.anchorAt(position, Qt.AlignLeft | Qt.AlignVCenter)
        position = QPointF(item.rect().left(),
                           item.rect().bottom() + item.size().height()/2.0)
        start += interval
        items.append(item)
        i += 1
    
    return items

def create_heading(slide, text, scene, pairs = None):

    if pairs:
        rect = FadingBoxItem(QRectF(slide.rect().topLeft() + QPointF(Width/8.0, Height/32.0),
                             QSizeF(Width * 0.75, Height/8.0)), pairs, scene)
    else:
        rect = RectangleItem(QRectF(slide.rect().topLeft() + QPointF(Width/8.0, Height/32.0),
                             QSizeF(Width * 0.75, Height/8.0)), scene)
    
    Title_Gradient.setStart(slide.top() + QPointF(0, Height/32.0))
    Title_Gradient.setFinalStop(slide.top() + QPointF(0, Height/32.0 + Height/8.0))
    rect.setColor(Title_Gradient)
    
    if pairs:
        title = FadingLabel(text, pairs, scene)
    else:
        title = LabelItem(text, scene)
    
    title.setFont(Title_Font)
    title.anchorAt(slide.top() + QPointF(0, Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
    
    return [rect, title]


# Slide creation class and methods

class Slides:

    def __init__(self, scene, view):
    
        self.scene = scene
        self.view = view
        self.slides = [
            self.title(QRectF(QPointF(0, 0), Slide_Size)),
            self.about_qt1(QRectF(QPointF(Width, 0), Slide_Size)),
            self.about_qt2(QRectF(QPointF(Width, 0), Slide_Size)),
            self.about_pyqt1(QRectF(QPointF(Width * 2, 0), Slide_Size)),
            self.about_pyqt2(QRectF(QPointF(Width * 2, 0), Slide_Size)),
            self.pyqt_modules1(QRectF(QPointF(Width * 3, 0), Slide_Size)),
            self.pyqt_modules2(QRectF(QPointF(Width * 3, 0), Slide_Size)),
            self.qtgui0(QRectF(QPointF(Width * 4, 0), Slide_Size)),
            self.qtgui1(QRectF(QPointF(Width * 4, 0), Slide_Size)),
            self.qtgui2(QRectF(QPointF(Width * 4, 0), Slide_Size)),
            self.qtgui_widgets0(QRectF(QPointF(Width * 5, 0), Slide_Size)),
            self.qtgui_widgets1(QRectF(QPointF(Width * 5, 0), Slide_Size)),
            self.qtgui_widgets2(QRectF(QPointF(Width * 5, 0), Slide_Size)),
            self.qtgui_widgets3(QRectF(QPointF(Width * 5, 0), Slide_Size)),
            self.qtgui_signals_and_slots(QRectF(QPointF(Width * 5, 0), Slide_Size)),
            self.qtgui_graphics_view1(QRectF(QPointF(Width * 6, 0), Slide_Size)),
            self.qtgui_graphics_view2(QRectF(QPointF(Width * 6, 0), Slide_Size)),
            #self.qtgui_graphics_view3(QRectF(QPointF(Width * 6, 0), Slide_Size)),
            self.phonon1(QRectF(QPointF(Width * 6, Height), Slide_Size)),
            self.phonon2(QRectF(QPointF(Width * 6, Height), Slide_Size)),
            self.phonon3(QRectF(QPointF(Width * 6, Height), Slide_Size)),
            self.qtxmlpatterns1(QRectF(QPointF(Width * 5, Height), Slide_Size)),
            self.qtxmlpatterns2(QRectF(QPointF(Width * 5, Height), Slide_Size)),
            self.qtxmlpatterns3(QRectF(QPointF(Width * 5, Height), Slide_Size)),
            self.qtwebkit1(QRectF(QPointF(Width * 4, Height), Slide_Size)),
            self.qtwebkit2(QRectF(QPointF(Width * 4, Height), Slide_Size)),
            self.qtwebkit3(QRectF(QPointF(Width * 4, Height), Slide_Size)),
            self.intermission(QRectF(QPointF(Width * 4, Height * 2), Slide_Size)),
            self.embedded_platforms0(QRectF(QPointF(Width * 5, Height * 2), Slide_Size)),
            self.embedded_platforms1(QRectF(QPointF(Width * 5, Height * 2), Slide_Size)),
            self.embedded_platforms2(QRectF(QPointF(Width * 5, Height * 2), Slide_Size)),
            self.embedded_platforms3(QRectF(QPointF(Width * 5, Height * 2), Slide_Size)),
            self.embedded_python0(QRectF(QPointF(Width * 6, Height * 2), Slide_Size)),
            self.embedded_python1(QRectF(QPointF(Width * 6, Height * 2), Slide_Size)),
            self.embedded_python2(QRectF(QPointF(Width * 6, Height * 2), Slide_Size)),
            self.embedded_python3(QRectF(QPointF(Width * 6, Height * 2), Slide_Size)),
            self.embedded_python4(QRectF(QPointF(Width * 6, Height * 2), Slide_Size)),
            self.embedded_qt_and_pyqt0(QRectF(QPointF(Width * 7, Height * 2), Slide_Size)),
            self.embedded_qt_and_pyqt1(QRectF(QPointF(Width * 7, Height * 2), Slide_Size)),
            self.embedded_qt_and_pyqt2(QRectF(QPointF(Width * 7, Height * 2), Slide_Size)),
            self.embedded_qt_and_pyqt3(QRectF(QPointF(Width * 7, Height * 2), Slide_Size)),
            self.embedded_qt_and_pyqt4(QRectF(QPointF(Width * 7, Height * 2), Slide_Size)),
            self.demonstrations1(QRectF(QPointF(Width * 8, Height * 2), Slide_Size)),
            self.demonstrations2(QRectF(QPointF(Width * 9, Height * 2), Slide_Size)),
            self.cutting_out_features0(QRectF(QPointF(Width * 10, Height * 2), Slide_Size)),
            self.cutting_out_features1(QRectF(QPointF(Width * 10, Height * 2), Slide_Size)),
            self.cutting_out_features2(QRectF(QPointF(Width * 10, Height * 2), Slide_Size)),
            self.finishing_up0(QRectF(QPointF(Width * 11, Height * 2), Slide_Size)),
            self.finishing_up1(QRectF(QPointF(Width * 11, Height * 2), Slide_Size)),
            self.finishing_up2(QRectF(QPointF(Width * 11, Height * 2), Slide_Size)),
            self.resources(QRectF(QPointF(Width * 11, Height * 3), Slide_Size))
            ]
        
        for number in range(len(self.slides) - 1):
        
            self.slides[number].addSlide(self.slides[number + 1])
    
    def tr(self, text):
    
        return QApplication.translate("Slides", text)
    
    def current(self):
    
        return self.slides[self.currentSlide]
    
    def next(self):
    
        try:
            return self.slides[self.currentSlide + 1]
        except IndexError:
            return 0
    
    def previous(self):
    
        try:
            return self.slides[self.currentSlide - 1]
        except IndexError:
            return 0
    
    def title(self, rect):
    
        slide = Slide(rect, self.scene)
        
        title = LabelItem("PyQt for Desktop and Embedded Devices", self.scene)
        title.setFont(Title_Font)
        title.anchorAt(QPointF(slide.center().x(), Height/4), Qt.AlignCenter)
        
        author = LabelItem("David Boddie", self.scene)
        author.setFont(Subtitle_Font)
        author.setColor(QColor(0, 128, 32))
        author.anchorAt(slide.center(), Qt.AlignHCenter | Qt.AlignBottom)
        
        email = LabelItem("<david.boddie@nokia.com>", self.scene)
        email.setFont(Email_Font)
        email.setColor(Qt.blue)
        email.anchorAt(slide.center(), Qt.AlignHCenter | Qt.AlignTop)
        
        location = HTMLItem(u"PyCon Tre, Firenze, 8<sup><small>th</small></sup>&ndash;10<sup><small>th</small></sup> May 2009", self.scene)
        location.setFont(Subtitle_Font)
        location.setColor(QColor(96, 64, 0))
        location.anchorAt(QPointF(slide.center().x(), 3*Height/4), Qt.AlignCenter)
        
        self.view.centerOn(slide.rect().center())
        
        slide.items = [title, author, location]
        return slide
    
    def about_qt1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        
        slide.items = create_heading(slide, "About Qt", self.scene)
        return slide
    
    def about_qt2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        slide.items = []
        position = QPointF(slide.rect().x() + Width / 8.0,
                           slide.rect().y() + Height / 4.0)
        
        list_items = (u"* Developed by Qt Software (Nokia)",
                      u"* Cross-platform C++ framework",
                      u"   * Linux, Windows, Mac OS X, other Unixes",
                      u"   * Embedded Linux, Windows CE, Series 60",
                      u"* Not just a widget toolkit \u2013 other features",
                      u"* Available under the GPL (version 3)",
                      u"* Available under the LGPL (version 2\u00b71)",
                      u"* Also available under a Commercial License")
        
        slide.items += create_list_items(list_items, position, self.scene)
        return slide
    
    def about_pyqt1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        
        slide.items = create_heading(slide, "About PyQt", self.scene)
        
        return slide
    
    def about_pyqt2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        slide.items = []
        position = QPointF(slide.rect().x() + Width / 8.0,
                           slide.rect().y() + Height / 4.0)
        
        list_items = (u"* Developed by Riverbank Computing Ltd.",
                      u"* Set of Python bindings to Qt",
                      u"   * Upcoming PyQt 4.5 will support Python 3",
                      u"* Bindings are generated using SIP",
                      u"* Includes most features of Qt",
                      u"* Available under the GPL (version 2 and 3)",
                      u"* Also available under a Commercial License")
        
        slide.items += create_list_items(list_items, position, self.scene)
        return slide
    
    def pyqt_modules1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        slide.items = create_heading(slide, "PyQt Modules", self.scene)
        
        label = LabelItem(u"PyQt exposes many of Qt's 21 modules.\n", self.scene)
        label.setColor(QColor(0, 0, 0))
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        colors = (QColor(192, 240, 192), QColor(192, 224, 224))
        x_positions = (slide.rect().x() + Width/4.0,
                       slide.rect().x() + Width/2.0,
                       slide.rect().x() + 3 * Width/4.0)
        y_position = slide.rect().y() + 3 * Height/8.0
        
        modules = ((u"QtCore", False),
                   (u"QtDesigner", False),
                   (u"QtGui", True),
                   (u"QtHelp", False),
                   (u"QtNetwork", False),
                   (u"QtOpenGL", False),
                   (u"QtScript", False),
                   (u"QtSql", False),
                   (u"QtSvg", False),
                   (u"QtWebKit", True),
                   (u"QtXmlPatterns", True),
                   ("Phonon", True))
        
        i = 0
        interval = 1.0 / ((len(modules) / 3) + 2)
        self.module_rects = {}
        
        for module, persistent in modules:
        
            start = -1.0 + ((i / 3) + (i % 3)) * interval
            
            if persistent:
                pairs = ((start, 0.0), (start + interval, 1.0))
            else:
                pairs = ((start, 0.0), (start + interval, 1.0), (0.0, 1.0), (1.0, 0.0))
            
            rect = FadingBoxItem(QRectF(0, 0, 7 * Width/32.0, Height/8.0),
                                 pairs, self.scene)
            rect.anchorAt(QPointF(x_positions[i % 3], y_position), Qt.AlignCenter)
            rect.setColor(colors[i % 2])
            self.module_rects[module] = rect
            
            label = LabelItem(module, self.scene)
            label.setFont(List_Font)
            label.setColor(QColor(64, 128, 64))
            label.setParentItem(rect)
            label.anchorAt(rect.boundingRect().center(), Qt.AlignCenter)
            
            slide.items.append(rect)
            slide.items.append(label)
            i += 1
            if i % 3 == 0:
                y_position += Height/8.0 + Height/32.0
        
        return slide
    
    def pyqt_modules2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"We'll take a quick look at these:",
                            ((-1, 0.0), (0, 1.0)), self.scene)
        label.setColor(QColor(0, 0, 0))
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
                          
        return slide
    
    def qtgui0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Graphical User Interfaces", self.scene)
        
        label = LabelItem(u"PyQt includes a comprehensive set of widgets:",
                          self.scene)
        label.setColor(QColor(0, 0, 0))
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        
        return slide
    
    def qtgui1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        left_column = -5*Width/16.0
        center_column = 0
        right_column = 5*Width/16.0
        
        checkBox = QCheckBox("C&ase sensitive")
        proxy = WidgetItem(checkBox, ((-1.0, 0.0), (-0.5, 1.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(left_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        comboBox = QComboBox()
        comboBox.addItem("Large (L)")
        proxy = WidgetItem(comboBox, ((-1.0, 0.0), (-0.5, 1.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(center_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        spinBox = QSpinBox()
        spinBox.setValue(12)
        proxy = WidgetItem(spinBox, ((-1.0, 0.0), (-0.5, 1.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(right_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        pushButton = QPushButton("Cancel")
        proxy = WidgetItem(pushButton, ((-0.75, 0.0), (-0.25, 1.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(left_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        slider = QSlider(Qt.Horizontal)
        slider.setValue(12)
        proxy = WidgetItem(slider, ((-0.75, 0.0), (-0.25, 1.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(center_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        radioButton = QRadioButton("Use existing")
        proxy = WidgetItem(radioButton, ((-0.75, 0.0), (-0.25, 1.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale, 0, 0, 0, 1.5 * Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(right_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(proxy)
        
        calendar = QCalendarWidget()
        calendar.setDateEditEnabled(False)
        proxy = WidgetItem(calendar, ((-0.5, 0.0), (0.0, 1.0)), self.scene)
        proxy.setTransform(QTransform(Widget_Scale, 0, 0, 0, Widget_Scale, 0, 0, 0, Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(left_column, 3*Height/4.0), Qt.AlignCenter)
        slide.items.append(proxy)
        
        tableWidget = QTableWidget(12, 2)
        tableWidget.verticalHeader().hide()
        tableWidget.horizontalHeader().setStretchLastSection(True)
        tableWidget.setHorizontalHeaderLabels(["Target", "Actual"])
        for row in range(12):
            for column in range(2):
                tableWidget.setItem(row, column, QTableWidgetItem(str(row * 4 - (row % 3) + (column % 2)*(1 - (row % 3)))))
                
        proxy = WidgetItem(tableWidget, ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(Widget_Scale, 0, 0, 0, Widget_Scale, 0, 0, 0, Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(center_column, 3*Height/4.0), Qt.AlignHCenter | Qt.AlignVCenter)
        slide.items.append(proxy)
        
        textEdit = QTextEdit()
        textEdit.setHtml("<h3>Rich Text Editor</h3><p><tt>QTextEdit</tt> is an "
                         "editor that can be used to display rich text "
                         "containing objects like these:</p>"
                         "<ul><li>lists <li>frames <li>tables <li>images</ul>")
        proxy = WidgetItem(textEdit, ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        proxy.setTransform(QTransform(Widget_Scale, 0, 0, 0, Widget_Scale, 0, 0, 0, Widget_Scale), True)
        proxy.anchorAt(slide.top() + QPointF(right_column, 3*Height/4.0), Qt.AlignHCenter | Qt.AlignVCenter)
        slide.items.append(proxy)
        
        checkBox = FadingLabel(u"QCheckBox", ((-1.0, 0.0), (-0.5, 1.0)), self.scene)
        checkBox.setFont(Class_Font)
        checkBox.anchorAt(slide.top() + QPointF(left_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(checkBox)
        
        comboBox = FadingLabel(u"QComboBox", ((-1.0, 0.0), (-0.5, 1.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        comboBox.setFont(Class_Font)
        comboBox.anchorAt(slide.top() + QPointF(center_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(comboBox)
        
        spinBox = FadingLabel(u"QSpinBox", ((-1.0, 0.0), (-0.5, 1.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        spinBox.setFont(Class_Font)
        spinBox.anchorAt(slide.top() + QPointF(right_column, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(spinBox)
        
        pushButton = FadingLabel(u"QPushButton", ((-0.75, 0.0), (-0.25, 1.0)), self.scene)
        pushButton.setFont(Class_Font)
        pushButton.anchorAt(slide.top() + QPointF(left_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(pushButton)
        
        slider = FadingLabel(u"QSlider", ((-0.75, 0.0), (-0.25, 1.0), (0.0, 1.0), (0.75, 0.0)), self.scene)
        slider.setFont(Class_Font)
        slider.anchorAt(slide.top() + QPointF(center_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(slider)
        
        radioButton = FadingLabel(u"QRadioButton", ((-0.75, 0.0), (-0.25, 1.0), (0.0, 1.0), (0.75, 0.0)), self.scene)
        radioButton.setFont(Class_Font)
        radioButton.anchorAt(slide.top() + QPointF(right_column, 17*Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(radioButton)
        
        calendar = FadingLabel(u"QCalendarWidget", ((-0.5, 0.0), (0.0, 1.0)), self.scene)
        calendar.setFont(Class_Font)
        calendar.anchorAt(slide.top() + QPointF(left_column, 7*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(calendar)
        
        tableWidget = FadingLabel(u"QTableWidget", ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        tableWidget.setFont(Class_Font)
        tableWidget.anchorAt(slide.top() + QPointF(center_column, 7*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(tableWidget)
        
        textEdit = FadingLabel(u"QTextEdit", ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        textEdit.setFont(Class_Font)
        textEdit.anchorAt(slide.top() + QPointF(right_column, 7*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(textEdit)
        
        Left_Margin = -9 * Code_Space_Width
        Top_Margin = 3*Height/8.0 - Code_Space_Height
        
        code = FadingPythonSource(
            'import sys\n'
            'from PyQt4.QtGui import *\n'
            '\n'
            'app = QApplication(sys.argv)\n'
            '\n'
            'checkBox = QCheckBox("C&ase sensitive")\n'
            'checkBox.show()\n'
            '\n'
            'pushButton = QPushButton("Cancel")\n'
            'pushButton.show()\n'
            '\n'
            'calendar = QCalendarWidget()\n'
            'calendar.show()\n'
            '\n'
            'sys.exit(app.exec_())\n',
            Large_Code_Font, ((-1.0, 0.0), (0.0, 0.0), (1.0, 1.0)), self.scene)
        code.anchorAt(slide.top() + QPointF(Left_Margin, Top_Margin), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(code)
        
        return slide
    
    def qtgui2(self, rect):
    
        slide = Slide(rect, self.scene)
        return slide
    
    def qtgui_widgets0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Widgets and Layouts", self.scene)
        
        return slide
    
    def qtgui_widgets1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        widget = pypi_information.PyPIWidget()
        
        proxy = WidgetItem(widget, ((-0.5, 0.0), (0.0, 1.0), (1.0, 1.0)), self.scene)
        t = QTransform()
        t = t.scale(1.25 * Widget_Scale, 1.25 * Widget_Scale)
        proxy.setTransform(t, True)
        proxy.anchorAt(slide.top() + QPointF(Width * 0.32, Height/4.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(proxy)
        
        for child in widget.children():
            if isinstance(child, QWidget):
                pos = QPointF(child.pos())
                child.setParent(None)
                pairs = ((-1.0, 0.0), (0.0, 0.0), (1.0, 1.0))
                child_proxy = WidgetItem(child, pairs, self.scene)
                child_proxy.setTransform(t, True)
                child_proxy.anchorAt(proxy.pos() + t.map(pos), Qt.AlignLeft | Qt.AlignTop)
                
                if isinstance(child, QComboBox):
                    child_proxy.setZValue(2)
                else:
                    child_proxy.setZValue(1.5)
                
                if isinstance(child, QTreeWidget):
                    self.qtgui_treeWidget = child_proxy
                else:
                    slide.items.append(child_proxy)
        
        Left_Margin = -Width/2.0 + Code_Space_Width
        Top_Margin = 5*Height/32.0
        
        code = FadingPythonSource(
            'class PyPIWidget(QWidget):\n'
            '  \n'
            '  def __init__(self, parent = None):\n'
            '  \n'
            '    QWidget.__init__(self, parent)\n',
            Code_Font, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        code.anchorAt(slide.top() + QPointF(Left_Margin, Top_Margin), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(code)
        
        code = FadingPythonSource(
            '    self.fieldCombo = QComboBox()\n'
            '    self.fieldCombo.addItems(["name", "version",\n'
            '        "author", "author_email", "maintainer",\n'
            '        "maintainer_email", "home_page",\n'
            '        "license", "summary", "description",\n'
            '        "keywords", "platform", "download_url"])\n'
            '    self.termsEdit = QLineEdit()\n'
            '    \n'
            '    layout = QFormLayout()\n'
            '    layout.addRow(self.tr("Search &field:"),\n'
            '                  self.fieldCombo)\n'
            '    layout.addRow(self.tr("Search &terms:"),\n'
            '                  self.termsEdit)\n',
            Code_Font, ((-1.0, 0.0), (0.0, 0.0), (1.0, 1.0)), self.scene)
        code.anchorAt(slide.top() + QPointF(Left_Margin,
            Top_Margin + Code_Space_Height*4), Qt.AlignLeft | Qt.AlignTop)
        
        slide.items.append(code)
        
        return slide
    
    def qtgui_widgets2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        Left_Margin = -Width/2.0 + Code_Space_Width
        Top_Margin = 5*Height/32.0
        
        code = FadingPythonSource(
            '    self.treeWidget = QTreeWidget()\n'
            '    self.treeWidget.setAlternatingRowColors(True)\n'
            '    self.treeWidget.setRootIsDecorated(False)\n'
            '    self.treeWidget.setHeaderLabels(\n'
            '        [self.tr("Name"), self.tr("Description")])\n'
            '    \n'
            '    mainLayout = QVBoxLayout()\n'
            '    mainLayout.addLayout(layout)\n'
            '    mainLayout.addWidget(self.treeWidget)\n',
            Code_Font, ((-1.0, 0.0), (0.0, 0.0), (1.0, 1.0)), self.scene)
        code.anchorAt(slide.top() + QPointF(Left_Margin,
            Top_Margin + Code_Space_Height * 16.5), Qt.AlignLeft | Qt.AlignTop)
        
        slide.items.append(self.qtgui_treeWidget)
        slide.items.append(code)
        return slide
    
    def qtgui_widgets3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        return slide
    
    def qtgui_signals_and_slots(self, rect):
    
        slide = Slide(rect, self.scene)
        
        Left_Margin = -Width/2.0 + Code_Space_Width
        Top_Margin = 5*Height/32.0
        
        code = FadingPythonSource(
            '    self.connect(self.termsEdit, SIGNAL("returnPressed()"), self.search)\n',
            Code_Font, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        code.anchorAt(slide.top() + QPointF(Left_Margin,
            Top_Margin + Code_Space_Height * 25), Qt.AlignLeft | Qt.AlignTop)
        
        slide.items.append(code)
        return slide
    
    def qtgui_graphics_view1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Graphics View", self.scene)
        
        label = LabelItem(u"The Graphics View framework provides a canvas:",
                          self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/3.0)
        
        list_items = (u"* Interactive items (drag and drop)",
                      u"* Nested items and groups",
                      u"* Animation",
                      u"* OpenGL rendering",
                      u"* Embedded widgets",
                      u"* Printing")
        
        slide.items += create_list_items(list_items, position, self.scene, 0.0)
        
        return slide
    
    def qtgui_graphics_view2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        nodes = NodesContainer(((-1.0, 0.0), (0.0, 1.0), (1.0, 0.25)), self.scene)
        nodes.setRect(QRectF(0, 0, min(Width/2.5, Height/2.5), Height/2.5))
        nodes.anchorAt(slide.center() + QPointF(Width/4.0, 0), Qt.AlignCenter)
        nodes.setPen(QPen(Qt.DashLine))
        slide.items.append(nodes)
        
        return slide
    
    def qtgui_graphics_view3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        return slide
    
    def phonon1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Multimedia Support", self.scene)
        
        label = LabelItem(u"Phonon handles audio and video:",
                          self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        
        return slide
        
    def phonon2(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        
        widget = player.Player()
        widget.resize(Width/3.0, Height/3.0)
        proxy = WidgetItem(widget, ((-1, 0.0), (0, 1.0)), self.scene)
        proxy.anchorAt(slide.left() + QPointF(Width/32.0, 0), Qt.AlignLeft | Qt.AlignVCenter)
        slide.items.append(proxy)
        
        code = FadingPythonSource(
            'class Player(QWidget):\n'
            '  \n'
            '  def __init__(self, parent = None):\n'
            '  \n'
            '    QWidget.__init__(self, parent)\n'
            '    \n'
            '    self.player = VideoPlayer(VideoCategory)\n'
            '    \n'
            '    # ...\n',
            Large_Code_Font, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        code.anchorAt(slide.center() + QPointF(-Code_Space_Width*8, -Code_Space_Height*6),
                      Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(code)
        
        return slide
    
    def phonon3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        code = FadingPythonSource(
            '  def play(self):\n'
            '  \n'
            '    if self.player.isPlaying():\n'
            '        self.player.stop()\n'
            '    else:\n'
            '        url = QUrl(self.urlEdit.text())\n'
            '        self.player.play(MediaSource(url))\n',
            Large_Code_Font, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        code.anchorAt(slide.center() + QPointF(-Code_Space_Width*8, Code_Space_Height*2),
                      Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(code)
        
        return slide
    
    def qtxmlpatterns1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "XML Processing", self.scene)
        
        label = LabelItem(u"Use XPath, XQuery and XSLT to process XML:",
                          self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        
        return slide
    
    def qtxmlpatterns2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        xml = FadingSource(
            '<sun rise="2009-05-01T05:32:17" set="2009-05-01T20:23:58" />\n'
            '<forecast>\n'
            '  <tabular>\n'
            '    <time from="2009-05-01T06:00:00" to="2009-05-01T12:00:00" period="1">\n'
            '      <symbol number="2" name="Fair" />\n'
            '      <precipitation value="0.0" />\n'
            '      <windDirection deg="114.6" code="ESE" name="East-southeast" />\n'
            '      <windSpeed mps="0.8" name="Light air" />\n'
            '      <temperature unit="celcius" value="8" />\n'
            '      <pressure unit="hPa" value="1022.2" />\n'
            '    </time>\n'
            '    ...\n',
            Code_Font, ((-1.0, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        xml.anchorAt(slide.top() + QPointF(-7 * Width/16.0, Height/4.0),
                     Qt.AlignLeft | Qt.AlignTop)
        xml.setFont(Code_Font)
        slide.items.append(xml)
        
        xq = FadingSource(
            'declare variable $url external;\n'
            '(\n'
            'string(doc($url)//sun/@rise),\n'
            'string(doc($url)//sun/@set),\n'
            ' \n'
            'for $time in doc($url)//tabular/time\n'
            'order by $time/@from\n'
            'return (string($time/@from),\n'
            '        string($time/symbol/@name), string($time/symbol/@number),\n'
            '        string($time/temperature/@value), string($time/temperature/@unit))\n'
            ')\n',
            Code_Font, ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        xq.anchorAt(slide.center() + QPointF(-7*Width/16.0, Height/8.0), Qt.AlignLeft | Qt.AlignTop)
        xq.setFont(Code_Font)
        xq.setDefaultTextColor(Qt.blue)
        slide.items.append(xq)
        
        return slide
    
    def qtxmlpatterns3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        widget = weatherquery.WeatherWidget()
        proxy = WidgetItem(widget, ((-1, 0.0), (0, 1.0)), self.scene)
        t = QTransform()
        t = t.scale(Widget_Scale, Widget_Scale)
        proxy.setTransform(t, True)
        proxy.anchorAt(slide.left() + QPointF(Width/16.0, Height/16.0),
                       Qt.AlignLeft | Qt.AlignVCenter)
        slide.items.append(proxy)
        
        code = FadingPythonSource(
            'def fetchForecast(self, place):\n'
            ' \n'
            '  url = "http://www.yr.no/place/" + \\\n'
            '        place + "/forecast.xml"\n'
            '  \n'
            '  b = QBuffer()\n'
            '  b.setData(query_string)\n'
            '  b.open(QBuffer.ReadOnly)\n'
            '  \n'
            '  query = QXmlQuery()\n'
            '  query.bindVariable("url",\n'
            '                     QXmlItem(QVariant(url)))\n'
            '  query.setQuery(b)\n'
            '  b.close()\n'
            '  \n'
            '  if query.isValid():\n'
            '  \n'
            '      self.results.clear()\n'
            '      query.evaluateTo(self.results)\n'
            '      self.updateTable()\n',
            Large_Code_Font, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        code.anchorAt(slide.center() + QPointF(-Code_Space_Width*8, -Code_Space_Height*7),
                      Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(code)
        
        label = FadingLabel(u"Could be useful if combined with GPS...",
                            ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.setColor(Qt.darkGreen)
        label.anchorAt(slide.bottom() + QPointF(0, -Height/32.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        return slide
    
    def qtwebkit1(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Web Browser Engine", self.scene)
        
        label = LabelItem(u"WebKit is integrated into Qt:", self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        
        return slide
    
    def qtwebkit2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/3.0)
        
        list_items = (u"* Web browser widget",
                      u"* JavaScript, SVG, CSS, SSL, etc.",
                      u"* Control over browser settings and history",
                      u"* Support for Netscape and native Qt plugins",
                      u"* Client-side storage",
                      u"* Support for in-page editing",
                      u"* Python/C++ objects can be added to pages")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 1.0, 0.0)
        
        return slide
    
    def qtwebkit3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        widget = simple_augmented.Browser()
        widget.resize(Width * 0.8, Height * 0.65)
        proxy = WidgetItem(widget, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        proxy.anchorAt(slide.center() + QPointF(0, Height/8.0), Qt.AlignCenter)
        slide.items.append(proxy)
        
        return slide
    
    def intermission(self, rect):
    
        slide = Slide(rect, self.scene)
        slide.items = create_heading(slide, "Intermission", self.scene)
        
        label = LabelItem(u"These features are nice on the desktop!", self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        
        label = LabelItem(u"We can also use them on embedded hardware.", self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.center(), Qt.AlignHCenter | Qt.AlignTop)
        
        return slide
    
    def embedded_platforms0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Embedded Platforms", self.scene)
        
        return slide
    
    def embedded_platforms1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Qt runs on Embedded Linux, Windows CE, Series 60",
                      u"* Python runs on Embedded Linux, Windows CE, Series 60",
                      u"* PyQt runs on Embedded Linux",
                      u"   * Windows CE?, Series 60?")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def embedded_platforms2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Disadvantages:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0+Width/16.0, Height/4.0 + Height/32.0), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"Advantages:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(Width/16.0, Height/4.0 + Height/32.0), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Small screens (\u2248 240 \u00d7 320)",
                      u"* Low memory (\u2248 64 MB)",
                      u"* Slow processors (\u2248 300 MHz)",
                      u"* Different architectures",
                      u"* Limited storage (\u2248 128MB)",
                      u"* Cut down environments")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        position = slide.top() + QPointF(Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Can be portable",
                      u"* Accelerometers",
                      u"* Touch screens",
                      u"* GPS",
                      u"* GSM, Wi-Fi, Bluetooth",
                      u"* Cameras")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def embedded_platforms3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Where is Embedded Linux used:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Phones, media players",
                      u"* GPS devices, Web tablets",
                      u"* Set top boxes",
                      u"* Routers, plug computing",
                      u"* Handhelds, toys, kit computing")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        label = FadingLabel(u"http://www.linuxdevices.com/", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(Email_Font)
        label.setColor(Qt.blue)
        label.anchorAt(slide.bottom() + QPointF(0, -Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        return slide
    
    def embedded_python0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Embedded Python", self.scene)
        
        return slide
    
    def embedded_python1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Nice things about Python (2.x):", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Portable C implementation",
                      u"* Fairly small (compared to all the Qt libraries)",
                      u"* Few dependencies",
                      u"* Batteries included")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def embedded_python2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Not so nice things about Python (2.x):", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Annoying to cross-compile (despite a good foundation)",
                      u"* Relies on a native interpreter at various points",
                      u"   * Runs setup.py using the built interpreter",
                      u"   * Needs a native interpreter to build a parser generator",
                      u"* Package-specific checks",
                      u"   * OpenSSL, Curses, pyexpat, Tkinter",
                      u"* Batteries included (even old ones)")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def embedded_python3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Ways to build Python for Linux Devices:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* OpenEmbedded", # \u2013 Gumstix, OpenMoko
                      u"* Scratchbox", # \u2013 Maemo
                      u"* Buildroot",
                      u"* Crosstool",
                      u"* Distribution packages (e.g., Debian)",
                      u"   ...")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        label = FadingLabel(u"Used to create toolchains and/or whole systems.", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.bottom() + QPointF(0, -Height/8.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        return slide
    
    def embedded_python4(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Ways I built Python for Linux Devices:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + 3 * Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Scratchbox",
                      u"   * Used to try PyQt on Maemo",
                      u"   * There are packages available now",
                      u"* Crosstool",
                      u"   * Used to try PyQt on a Greenphone")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        return slide
    
    def embedded_qt_and_pyqt0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Qt and PyQt on Linux Devices", self.scene)
        
        return slide
    
    def embedded_qt_and_pyqt1(self, rect):
    
        slide = Slide(rect, self.scene)
        label = FadingLabel(u"Which graphics system to use?", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/4.0 + Height/32.0)
        
        list_items = (u"X11 (Qt for X11):",
                      u"* Fairly standard procedure for building Qt and PyQt",
                      u"* Not all that common to cross-compile Qt for X11",
                      u"* Qt works better with X extensions (Render)",
                      u"* Develop using PyQt for X11",
                      u" ",
                      u"QWS (Qt for Embedded Linux):",
                      u"* Fairly easy to build Qt for Embedded Linux",
                      u"* PyQt needs patching for cross-compilation",
                      u"* Qt uses the framebuffer",
                      u"* Develop using PyQt derived from PyQt for X11")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def embedded_qt_and_pyqt2(self, rect):
    
        slide = Slide(rect, self.scene)
        label = FadingLabel(u"Developing and Simulating", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"X11-based devices:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, Height/4.0 + Height/32.0), Qt.AlignLeft | Qt.AlignVCenter)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0 - Height/32.0)
        
        list_items = (u"* Use Xephyr (nested X server) to simulate a small screen",
                      u"* Run or simulate the device's window system",
                      u"* The Maemo SDK emulates the device environment",
                      u"* Possible to use system PyQt to prototype applications",
                      u"    * Beware of version differences")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        Code_Top_Left = slide.left() + QPointF(Width/16.0, Height/4.0)
        
        rect = FadingBoxItem(QRectF(0, 0, 7*Width/8.0, Height/8.0),
                             ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        rect.anchorAt(Code_Top_Left, Qt.AlignLeft | Qt.AlignTop)
        rect.setColor(Qt.darkBlue)
        slide.items.append(rect)
        
        shell = FadingSource(
            'Xephyr -ac -extension Composite -screen 800x480 :1\n'
            'DISPLAY=:1 python application.py',
            Source_Font, ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        shell.anchorAt(Code_Top_Left, Qt.AlignLeft | Qt.AlignTop)
        shell.setFont(Source_Font)
        shell.setDefaultTextColor(Qt.white)
        shell.setZValue(2)
        slide.items.append(shell)
        
        return slide
    
    def embedded_qt_and_pyqt3(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"QWS-based devices:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, Height/4.0 + Height/32.0), Qt.AlignLeft | Qt.AlignVCenter)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0 - Height/32.0)
        
        list_items = (u"* Use a virtual framebuffer or VNC to simulate the screen",
                      u"* Run or simulate the device's window system",
                      u"* No need for a dedicated window manager",
                      u"* Need to build your own libraries for desktop and device")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        Code_Top_Left = slide.left() + QPointF(Width/16.0, Height/4.0)
        
        rect = FadingBoxItem(QRectF(0, 0, 7*Width/8.0, Height/8.0),
                             ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        rect.anchorAt(Code_Top_Left, Qt.AlignLeft | Qt.AlignTop)
        rect.setColor(Qt.darkGreen)
        slide.items.append(rect)
        
        shell = FadingSource(
            'python application.py -qws -display VNC:0:size=240x320\n'
            'vncviewer :0',
            Source_Font, ((-0.5, 0.0), (0.0, 1.0), (1.0, 0.0)), self.scene)
        shell.anchorAt(Code_Top_Left, Qt.AlignLeft | Qt.AlignTop)
        shell.setFont(Source_Font)
        shell.setDefaultTextColor(Qt.white)
        shell.setZValue(2)
        slide.items.append(shell)
        
        return slide
    
    def embedded_qt_and_pyqt4(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Common approach:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, Height/4.0 + Height/32.0), Qt.AlignLeft | Qt.AlignVCenter)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0)
        
        list_items = (u"* Applications can be developed on desktops using PyQt",
                      u"* Obviously, look and feel may be a bit different",
                      u"* The APIs should be the same",
                      u"* Care must be taken with the widgets used:",
                      u"   * Input widgets without a keyboard...",
                      u"   * Scrollbars on a small screen...")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        return slide
    
    def demonstrations1(self, rect):
    
        slide = Slide(rect, self.scene)
        slide.items = create_heading(slide, "Demonstrations", self.scene)
        
        label = FadingLabel(u"Two devices:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/8.0 + Height/32.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        Left_Screen = slide.center() + QPointF(-Width/4.0, 0)
        
        label = FadingLabel(u"Nokia N800:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(Left_Screen + QPointF(0, -Height/32.0 - 0.25 * Widget_Scale * 480), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        position = slide.left() + QPointF(Width/16.0, Height/16.0 + 0.25 * Widget_Scale * 480)
        
        list_items = (u"CPU: 400 MHz OMAP2420",
                      u"Screen: 800 \u00d7 480")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        t = QTransform()
        t = t.scale(0.5 * Widget_Scale, 0.5 * Widget_Scale)
        
        rect = FadingBoxItem(QRectF(0, 0, 800, 480),
                             ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        rect.setTransform(t, True)
        rect.anchorAt(Left_Screen, Qt.AlignCenter)
        rect.setPen(QPen(Qt.DashLine))
        slide.items.append(rect)
        
        pixmap = QPixmap(os.path.join(This_Dir, "Images/photos-n800.png"))
        pixmap = pixmap.transformed(t, Qt.SmoothTransformation)
        item = QGraphicsPixmapItem(pixmap)
        item.setPos(Left_Screen + QPointF(-pixmap.width()/2, -pixmap.height()/2))
        self.scene.addItem(item)
        
        Right_Screen = slide.center() + QPointF(Width/4.0, 0)
        
        label = FadingLabel(u"Greenphone:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(Right_Screen + QPointF(0, -Height/32.0 - 0.25 * Widget_Scale * 480), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        position = slide.center() + QPointF(Width/16.0, Height/16.0 + 0.25 * Widget_Scale * 480)
        
        list_items = (u"CPU: 312 MHz Intel PXA270",
                      u"Screen: 240 \u00d7 320")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        t = QTransform()
        t = t.scale(0.5 * Widget_Scale, 0.5 * Widget_Scale)
        
        rect = FadingBoxItem(QRectF(0, 0, 240, 320),
                             ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        rect.setTransform(t, True)
        rect.anchorAt(Right_Screen, Qt.AlignCenter)
        rect.setPen(QPen(Qt.DashLine))
        slide.items.append(rect)
        
        pixmap = QPixmap(os.path.join(This_Dir, "Images/photos-greenphone.png"))
        pixmap = pixmap.transformed(t, Qt.SmoothTransformation)
        item = QGraphicsPixmapItem(pixmap)
        item.setPos(Right_Screen + QPointF(-pixmap.width()/2, -pixmap.height()/2))
        self.scene.addItem(item)
        
        return slide
    
    def demonstrations2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = LabelItem(u"At this point we show some demonstrations...", self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/4.0), Qt.AlignHCenter | Qt.AlignTop)
        
        pixmap = QPixmap(os.path.join(This_Dir, "Images/demos-faded.jpg"))
        pixmap = pixmap.scaled(0.5 * Widget_Scale * pixmap.width(),
                               0.5 * Widget_Scale * pixmap.height(),
                               Qt.KeepAspectRatio, Qt.SmoothTransformation)
        item = QGraphicsPixmapItem(pixmap)
        item.setPos(slide.center() + QPointF(-pixmap.width()/2, Height/8.0 - pixmap.height()/2))
        self.scene.addItem(item)
        
        return slide
    
    def cutting_out_features0(self, rect):
    
        slide = Slide(rect, self.scene)
        slide.items = create_heading(slide, "Cutting Out Features", self.scene)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0 + Height/32.0)
        
        list_items = (u"On desktops, we want as many features as possible...",
                      u"...but...",
                      u"...on embedded devices, we might not want everything.")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def cutting_out_features1(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Inappropriate features:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, 3*Height/16.0), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/4.0 + Height/32.0)
        
        list_items = (u"* Classic dialogs too large for small screen devices",
                      u"* Menu bars, dock windows follow the wrong paradigm")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        label = FadingLabel(u"Unnecessary features:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, 3*Height/8.0 + Height/32.0), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/2.0)
        
        list_items = (u"* Devices with touch screens don't need cursors",
                      u"* Specialized displays don't need all the widget styles")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        label = FadingLabel(u"Redundant features:", ((-1.0, 0.0), (0.0, 1.0), (0.5, 0.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(-Width/2.0 + Width/16.0, 5*Height/8.0), Qt.AlignLeft | Qt.AlignTop)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/4.0 - Height/32.0)
        
        list_items = (u"* We don't really need:",
                      u"   * four sets of XML classes,",
                      u"   * two JavaScript engines,",
                      u"   * two ways to access networked resources",)
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0, 0.5, 0.0)
        
        return slide
    
    def cutting_out_features2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Configuring Qt for Embedded Linux:", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/4.0 - Height/32.0), Qt.AlignCenter)
        slide.items.append(label)
        
        # Printing a FadingPixmap failed for some reason, so use a widget
        # instead.
        widget = QLabel()
        pixmap = QPixmap(os.path.join(This_Dir, "Images/qconfig.png"))
        widget.setPixmap(pixmap)
        widget.resize(pixmap.size())
        item = WidgetItem(widget, ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        t = QTransform()
        t = t.scale(Widget_Scale, Widget_Scale)
        item.setTransform(t, True)
        item.anchorAt(slide.center() + QPointF(0, Height/16.0), Qt.AlignCenter)
        slide.items.append(item)
        
        label = FadingLabel(u"QConfig lets you remove features from an embedded build", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.bottom() + QPointF(0, -Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        return slide
    
    def finishing_up0(self, rect):
    
        slide = Slide(rect, self.scene, printable = False)
        slide.items = create_heading(slide, "Finishing Up", self.scene)
        
        return slide
    
    def finishing_up1(self, rect):
    
        slide = Slide(rect, self.scene)
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, 3*Height/8.0 + Height/32.0)
        
        label = FadingLabel(u"PyQt makes it possible to", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 7*Height/32.0), Qt.AlignCenter)
        slide.items.append(label)
        
        position = slide.top() + QPointF(-Width/2.0+Width/16.0, Height/4.0 + Height/32.0)
        
        list_items = (u"* Write code that works on different (embedded) platforms",
                      u"* Despite allowances for differences between devices",
                      u"   * You get to work around those at a high level",
                      u"* Prototype applications on the desktop")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        return slide
    
    def finishing_up2(self, rect):
    
        slide = Slide(rect, self.scene)
        
        label = FadingLabel(u"Python makes it possible to", ((-1.0, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.center() + QPointF(0, Height/16.0), Qt.AlignCenter)
        slide.items.append(label)
        
        position = slide.center() + QPointF(-Width/2.0+Width/16.0, Height/8.0)
        
        list_items = (u"* Ignore problems like cross-compiling",
                      u"   * Build on top of existing pure-Python code",
                      u"* Write portable, deployable applications",
                      u"* Take advantage of interactivity to prototype on devices")
        
        slide.items += create_list_items(list_items, position, self.scene, -1.0)
        
        return slide
    
    def resources(self, rect):
    
        slide = Slide(rect, self.scene)
        slide.items = create_heading(slide, "Resources", self.scene)
        
        label = FadingLabel(u"Qt", ((-1.0, 0.0), (-0.6, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/4.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        label = FadingLabel(u"http://www.qtsoftware.com/", ((-1.0, 0.0), (-0.6, 1.0)), self.scene)
        label.setFont(Email_Font)
        label.setColor(Qt.blue)
        label.anchorAt(slide.top() + QPointF(0, Height/4.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"PyQt", ((-0.8, 0.0), (-0.4, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        label = FadingLabel(u"http://www.riverbankcomputing.com/", ((-0.8, 0.0), (-0.4, 1.0)), self.scene)
        label.setFont(Email_Font)
        label.setColor(Qt.blue)
        label.anchorAt(slide.top() + QPointF(0, 3*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"PyQt and PyKDE Wiki", ((-0.7, 0.0), (-0.3, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, Height/2.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        label = FadingLabel(u"http://www.diotavelli.net/PyQtWiki/", ((-0.7, 0.0), (-0.3, 1.0)), self.scene)
        label.setFont(Email_Font)
        label.setColor(Qt.blue)
        label.anchorAt(slide.top() + QPointF(0, Height/2.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"Develer", ((-0.4, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.anchorAt(slide.top() + QPointF(0, 5*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        label = FadingLabel(u"http://www.develer.com/", ((-0.4, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(Email_Font)
        label.setColor(Qt.blue)
        label.anchorAt(slide.top() + QPointF(0, 5*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        label = FadingLabel(u"Rapid GUI Programming with Python and Qt", ((-0.3, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.setColor(Qt.darkGreen)
        label.anchorAt(slide.top() + QPointF(0, 6*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignBottom)
        slide.items.append(label)
        
        label = FadingLabel(u"by Mark Summerfield", ((-0.3, 0.0), (0.0, 1.0)), self.scene)
        label.setFont(List_Font)
        label.setColor(Qt.darkGreen)
        label.anchorAt(slide.top() + QPointF(0, 6*Height/8.0 + Height/16.0), Qt.AlignHCenter | Qt.AlignTop)
        slide.items.append(label)
        
        return slide

    
def init_slides(scene, view):

    global Width, Height, Title_Gradient, Slide_Size
    
    Width, Height, Title_Gradient, Slide_Size = init_dimensions(
        view.width(), view.height())
    
    global Title_Font, Subtitle_Font, List_Font, Class_Font, Email_Font
    global Source_Font, Code_Font, Large_Code_Font
    global Code_Space_Width, Code_Space_Height
    
    Title_Font, Subtitle_Font, List_Font, Class_Font, Email_Font, \
        Source_Font, Code_Font, Large_Code_Font, Code_Space_Width, \
        Code_Space_Height = init_fonts(view.logicalDpiY(), Height)
    
    global Widget_Scale
    
    Widget_Scale = min(Width/1024.0, Height/768.0)
    
    # Find examples
    import sys
    this_file = QFileInfo(__file__)
    d = this_file.dir()
    d.cdUp()
    d.cd("Examples")
    sys.path.append(str(d.path()))
    
    global photos_n800, photos_rotated, player, pypi_information
    global simple_augmented, weatherquery
    from XMLPatterns import weatherquery
    from Phonon import player
    from Widgets import pypi_information
    from WebKit import simple_augmented
    from WebKit import simple_augmented
    
    slides = Slides(scene, view)
    return slides
